/* Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/**************************************************************************
 * @file        Sensorless_Trap_Parameters_Setup.h
 * @author      MDBU Software Team
 * @brief       SPI API file for SPI Module.
 * @note        Copyright (c) 2016 Texas Instruments Incorporated.
 *              All rights reserved.
 * @date		May 2016
 ******************************************************************************/

// USER DEFINES
#define ALGO_ID						2       // 2 Indicates sensorless , 1 indicates sensored algorithms
#define PROD_ID                     1       // 1 Indicated AMD PL devices
#define FW_VER_MAJ                  1       // FW version major
#define FW_VER_MIN                  0       // FW version minor
#define FW_VER_PATCH                1       // FW version patch

//SYSTEM Parameters
#define PWM_FREQUENCY				25U   		// PWM switching frequency in KHz
#define MOTOR_RATED_VOLTAGE			24.0F
#define MOTOR_RATED_SPEED 			4000.0F       // 3200 RPM Annaheim motor ,
#define MOTOR_POLES					8         // 8 pole for Annaheim motor

#define ALIGN_OR_IPD				0       //  0 indicates Six pulse method , 1 indicates rotor align
//ISC User Parameters
#define ISC_THRESHOLD_MAX_SPEED 	100		// Maximim Speed above which ISC routine
#define ISC_THRESHOLD_MIN_SPEED 	50		// Minimum Speed below which ISC routine skips to IPD after brake time set by ISC BRAKE TIME in ms to disscipate the residual BEMF( 1.4v *4095)/57.5 = 100
#define ISC_MIN_LINE_BEMF           20      // Minimum Line BEMF below which motor is assumed to be stand still
#define ISC_ZEROTH_PHASE_MATCH      12      // After two electrical cycles or 12 phase crossings ( for 1 electrical cycles, 3 phases cross each other 6 times) of free wheeling Start sampling the BEMF for better accuracy
#define ISC_FIRST_PHASE_MATCH       13      // After two electrical cycles or 13 phase crossings ( for 1 electrical cycles, 3 phases cross each other 6 times) of free wheeling sample the first BEMF match or phase crossing
#define ISC_SECOND_PHASE_MATCH      14      // After two electrical cycles or 14 phase crossings ( for 1 electrical cycles, 3 phases cross each other 6 times) of free wheeling sample the second BEMF match or phase crossing

//IPD User Parameters
#define IPD_ADD_BRAKE	 			30		// the (IPDRiseTime/512)+this number  IPDRiseTime is measured using timerA at 12.5MHz so 400us = 5000
#define IPD_PULSE_TIME	 			2500		// set no of clock cycles (1 clock cycles = 40nS) a voltage pulse is applied on a phase for Initial Position detection by Six pulse method
#define	IPD_DECAY_CONSTANT 			3		// how many times longer to coast the motor and wait until next pulse after braking in IPD

//Align User Parameters
#define ALIGN_SECTOR				1		// Align commutation sequence (1-6)
#define ALIGN_WAIT_TIME				500		// Number of PWM cycles to Wait during align align time seconds = [this number * 41us]

//Open Loop Acceleration User Parameters
#define ACCEL_RATE 					10		// {Hz/s}
#define ACCEL_STOP 					40	// {Hz}
#define ACCEL_VELOCITY_INIT			10   // {Hz}
#define MOTOR_UNDER_RATING			120  // percentage of rated voltage at which motor spins at rated speed for applied load , set to 120 to start at loaded conditions

//Closed Loop User Parameters
#define BEMF_THRESHOLD 				1550// BEMF Integration threshold according to calculations of BEMF waveform
#define RAMP_RATE_DELAY		 		100		// This number controls the acceleration,  duty cycle is updated after ( RAMP_RATE_DELAY * 1000) clock cycles
#define RAMP_RATE					1		// This is the change in dutycycle (increment/decrement) for every update
#define COMMUTATION_BLANK_TIME 		5		// How many PWM cycles to blank before sampling the BEMF
#define PWM_BLANK_COUNTS 			5		// How many Clock cycles before the center of PWM  the BEMF is sampled

#define MAX_DUTY_CYCLE 				1000		// relative to PWM_PERIOD
#define MIN_OFF_DUTY 				240			// relative to PWM_PERIOD
#define MIN_ON_DUTY 				250			// relative to PWM_PERIOD
#define PWM_FACTOR					0       	//  ADC 12 bit to PWM width ratio , by default 0 represents 12 bit scaling

/* Fault handling setup */						/*ADC Max ref voltage is 3.3v ,  VCC is scaled by 0.0573 internally so that VCC ref input to ADC never cross 3.3v The maximum supply voltage is 115V.*/
#define UNDER_VOLTAGE_LIMIT (8)			    /* Under Voltage set for below the specified volts - digital values are there by cal as (Limit*4095)/Full scale voltage  */
#define OVER_VOLTAGE_LIMIT	(30)     			/* Over Voltage set for above the specified volts - digital values are there by cal as (Limit*4095)/Full scale voltage */
#define MIN_POWER_SUPPLY    (5)                 /* Minimum power supply required to access the SPI , when supply voltage is below this limit gate drivers are disabled */
#define STALLDETECT_REV_THRESHOLD	(1)			/* Number of revolutions below which stall fault will be flagged */
#define STALLDETECT_TIMER_THRESHOLD (200)		/* Time in milli seconds above which if motor doesnt spin min revolutions specified above(STALLDETECT_REV_THRESHOLD) a stall fault is triggered */
#define MOTOR_PHASE_CURRENT_LIMIT (868)        /* Defines the max allowed motor phase current in digital counts . Motor phase current is monitored every electrical cycle , and when ever current limit is reached an OC fault is Triggered */
#define AUTO_FAULT_RECOVERY_TIME (25000)      	/*  Delay in milli Seconds after which system reinitialises itself if fault gets cleared */

