/**************************************************************************
 * @file        global.c
 * @author      MDBU Software Team
 * @brief       global functions used for application
 * @note        Copyright (c) 2016 Texas Instruments Incorporated.
 *              All rights reserved.
 ******************************************************************************/
#include "drv8343.h"
#include "drv8x_HAL.h"
#include "SPI_API.h"
// Registers

FLT_STAT_Obj Fault_Status_Reg;
DIAG_STAT_A_Obj Diag_status_A_Reg;
DIAG_STAT_B_Obj Diag_status_B_Reg;
DIAG_STAT_C_Obj Diag_status_C_Reg;
DRV_CTRL_IC1_Obj Drv_Control_Reg1;
DRV_CTRL_IC2_Obj Drv_Control_Reg2;
DRV_CTRL_IC3_Obj Drv_Control_Reg3;
DRV_CTRL_IC4_Obj Drv_Control_Reg4;
DRV_CTRL_IC5_Obj Drv_Control_Reg5;
DRV_CTRL_IC6_Obj Drv_Control_Reg6;
DRV_CTRL_IC7_Obj Drv_Control_Reg7;
DRV_CTRL_IC8_Obj Drv_Control_Reg8;
DRV_CTRL_IC9_Obj Drv_Control_Reg9;
DRV_CTRL_IC10_Obj Drv_Control_Reg10;
DRV_CTRL_IC11_Obj Drv_Control_Reg11;
DRV_CTRL_IC12_Obj Drv_Control_Reg12;
DRV_CTRL_IC13_Obj Drv_Control_Reg13;
DRV_CTRL_IC14_Obj Drv_Control_Reg14;

REG_MAP_Obj Reg_Map_Cache;
DRV_Control_Obj DRV_Control;

uint16_t Register_Counter;

uint16_t DRV8343S_Idrive_RegData[NUMCONFIG_IDRIVE_VALUES] = {DRV8343S_IdriveP_MODE0, DRV8343S_IdriveP_MODE1, DRV8343S_IdriveP_MODE2, DRV8343S_IdriveP_MODE3,
															DRV8343S_IdriveP_MODE4, DRV8343S_IdriveP_MODE5, DRV8343S_IdriveP_MODE6, DRV8343S_IdriveP_MODE7,
															DRV8343S_IdriveP_MODE8, DRV8343S_IdriveP_MODE9, DRV8343S_IdriveP_MODE10, DRV8343S_IdriveP_MODE11,
															DRV8343S_IdriveP_MODE12, DRV8343S_IdriveP_MODE13, DRV8343S_IdriveP_MODE14, DRV8343S_IdriveP_MODE15 };

uint16_t DRV8343S_Tdrive_RegData[NUMCONFIG_TDRIVE_VALUES] = {DRV8343S_Tdrive_MODE0, DRV8343S_Tdrive_MODE1, DRV8343S_Tdrive_MODE2, DRV8343S_Tdrive_MODE3};
/*function
 * drv83xx_is_CSA_BI_DIR_Mode(void)
 * This function when called returns if the device is in Bidirectional CSA configuration or Uni Direction CSA configuration
 * */

unsigned char drv83xx_is_CSA_BI_DIR_Mode(void)
{
	unsigned int regValue;

		regValue = drv83xx_cachetoReg(SPI_REG_DRV_CTRL_13);  // Read the Register cache that holds the CSA mode settings
		return (regValue & DRV8343S_VREF_DIV_MASK);          // Return if the Vref_div bit is High (Bi directional ) or ) (Uni Dirction)
}

/*function
 * drv83xx_set_Six_PWM_Mode(void)
 * This function when called sets the mode to six PWM mode
 * */

void drv83xx_set_Six_PWM_Mode(void)
{
	unsigned int regValue;

		regValue = drv83xx_cachetoReg(SPI_REG_DRV_CTRL_1);
		regValue &= ~DRV8343S_PWM_MODE_MASK;                  // Six PWM Mode

		drv83xx_registerWrite(SPI_REG_DRV_CTRL_1, regValue);        // Write back the updated value to the cache and SPI

}
/*function
 * drv83xx_set_One_PWM_Mode(void)
 * This function when called sets the mode to one PWM mode
 * */

void drv83xx_set_One_PWM_Mode(void)
{
	unsigned int regValue;

	regValue = drv83xx_cachetoReg(SPI_REG_DRV_CTRL_1);

		regValue &= ~DRV8343S_PWM_MODE_MASK;
		regValue |= DRV8343S_PWM_MODE_1x_MASK;
		drv83xx_registerWrite(SPI_REG_DRV_CTRL_1, regValue);        // Write back the updated value to the cache and SPI
}
/*function
 * drv83xx_set_IND_ABC_PWM_Mode(void)
 * This function when called sets the mode to one PWM mode
 * */

void drv83xx_set_IND_ABC_PWM_Mode(void)
{
	unsigned int regValue;

	regValue = drv83xx_cachetoReg(SPI_REG_DRV_CTRL_1);

		regValue &= ~DRV8343S_PWM_MODE_MASK;
		regValue |= DRV8343S_BRIDGE_ABC_MODE_MASK;
		drv83xx_registerWrite(SPI_REG_DRV_CTRL_1, regValue);        // Write back the updated value to the cache and SPI
}
/*function
 * drv83xx_set_IND_BRIDGE_AB_FET_C_PWM_Mode(void)
 * This function when called sets the mode to one PWM mode
 * */

void drv83xx_set_IND_AB_FET_C_PWM_Mode(void)
{
	unsigned int regValue;

	regValue = drv83xx_cachetoReg(SPI_REG_DRV_CTRL_1);

		regValue &= ~DRV8343S_PWM_MODE_MASK;
		regValue |= DRV8343S_BRIDGE_AB_FET_C_MODE_MASK;
		drv83xx_registerWrite(SPI_REG_DRV_CTRL_1, regValue);        // Write back the updated value to the cache and SPI
}
/*function
 * drv83xx_set_IND_BC_FET_A_PWM_Mode(void)
 * This function when called sets the mode to one PWM mode
 * */

void drv83xx_set_IND_BC_FET_A_PWM_Mode(void)
{
	unsigned int regValue;

	regValue = drv83xx_cachetoReg(SPI_REG_DRV_CTRL_1);

		regValue &= ~DRV8343S_PWM_MODE_MASK;
		regValue |= DRV8343S_BRIDGE_BC_FET_A_MODE_MASK;
		drv83xx_registerWrite(SPI_REG_DRV_CTRL_1, regValue);        // Write back the updated value to the cache and SPI
}
/*function
 * drv83xx_set_IND_A_FET_BC_PWM_Mode(void)
 * This function when called sets the mode to one PWM mode
 * */

void drv83xx_set_IND_A_FET_BC_PWM_Mode(void)
{
	unsigned int regValue;

	regValue = drv83xx_cachetoReg(SPI_REG_DRV_CTRL_1);

		regValue &= ~DRV8343S_PWM_MODE_MASK;
		regValue |= DRV8343S_BRIDGE_A_FET_BC_MODE_MASK;
		drv83xx_registerWrite(SPI_REG_DRV_CTRL_1, regValue);        // Write back the updated value to the cache and SPI
}
/*function
 * drv83xx_set_FET_ABC_PWM_Mode(void)
 * This function when called sets the mode to one PWM mode
 * */

void drv83xx_set_FET_ABC_PWM_Mode(void)
{
	unsigned int regValue;

	regValue = drv83xx_cachetoReg(SPI_REG_DRV_CTRL_1);

		regValue &= ~DRV8343S_PWM_MODE_MASK;
		regValue |= DRV8343S_FET_ABC_MODE_MASK;
		drv83xx_registerWrite(SPI_REG_DRV_CTRL_1, regValue);        // Write back the updated value to the cache and SPI
}

/*function
 * drv83xx_ClearFault_Status(void)
 * This function when called clears the Fault bit by which all the warnings and faults are cleared
 * */


void drv83xx_ClearFault_Status(void)
{
	drv8343_SPI_Write(SPI_REG_DRV_CTRL_1,(drv8343_SPI_Read(SPI_REG_DRV_CTRL_1) | DRV8343S_CLR_FLT_MASK));          //Try to Clear the faults and warnings
}

/*function
 * drv83xx_Start_OLP_Test(void)
 * This function when called clears the Fault bit by which all the warnings and faults are cleared
 * */


void drv83xx_Start_OLP_Test(void)
{
	drv8343_SPI_Write(SPI_REG_DRV_CTRL_2,(drv8343_SPI_Read(SPI_REG_DRV_CTRL_2) | DRV8343S_EN_OLP_MASK));          //Try to Clear the faults and warnings
}
/*function
 * drv83xx_Control_Init(void)
 * This function initializes the initial values of the SPI settings required for the application
 * */
unsigned int drv83xx_getFault_Status(void)
{
	unsigned int faultStatus = drv8343_SPI_Read(SPI_REG_FAULT_STAT);     // Check fault register;

	if(faultStatus & DRV8343S_FAULT_MASK)                                          // If it is Fault
	{
		if(faultStatus & DRV8343S_OCP_MASK)
		{
			return  OVER_CURRENT_FAULT;
		}
		else if(faultStatus & DRV8343S_UVLO_MASK)
		{
			return UNDER_VOLTAGE_LOCK_OUT_FAULT;
		}
		else if(faultStatus & DRV8343S_OTSD_MASK)
		{
			return OVER_TEMPERATURE_FAULT;
		}
		else if(faultStatus & DRV8343S_GDF_MASK)
		{
			return GATE_DRIVE_FAULT;
		}
		else if (faultStatus & DRV8343S_CPUV_MASK)
		{
			return CHARGE_PUMP_UV_FAULT;
		}
		else if (faultStatus & DRV8343S_OL_SHT_MASK)
		{
			return OPEN_LOAD_SHORT_FAULT;
		}
		else
		{
			return UNKNOWN_FAULT;
		}

	}
	else         // If the Fault pin is toggling because of warnings then clear the warnings
	{
       	drv83xx_ClearFault_Status();							   // Try to Clear the faults and warnings
       	return NO_FAULT;
	}
}
void drv83xx_Control_Init(void)
{
	DRV_Control.Idrive_Min_Value = (GATE_TO_DRAIN_CHARGE * 1000 / RISE_TIME);    // The Idrive calculation Can be seen in datasheet
	DRV_Control.Tdrive_Max_Value = (2* MAX(RISE_TIME,FALL_TIME));                 // The Tdrive value is calculated based on datasheet , For DRv83xxS devices Tdrive is twice of Rise time/ Fall time

	for(DRV_Control.Register_Counter = 0; DRV_Control.Register_Counter < NUMCONFIG_IDRIVE_VALUES ; DRV_Control.Register_Counter++)
	{
		if(DRV_Control.Idrive_Min_Value >= DRV8343S_Idrive_RegData[DRV_Control.Register_Counter])
		{
			DRV_Control.Idrive_Setting = DRV_Control.Register_Counter;
		}
	}
	for(DRV_Control.Register_Counter = 0; DRV_Control.Register_Counter < NUMCONFIG_TDRIVE_VALUES ; DRV_Control.Register_Counter++)
	{
		if(DRV_Control.Tdrive_Max_Value >= DRV8343S_Tdrive_RegData[DRV_Control.Register_Counter])
		{
			DRV_Control.Tdrive_Setting = DRV_Control.Register_Counter;
		}
	}

}
void drv83xx_Register_Read(void)
{
    uint16_t regValue;

    // Read Register 0x00
    regValue = drv8343_SPI_Read(SPI_REG_FAULT_STAT);

    Fault_Status_Reg.FSO_FAULT = (regValue & DRV8343S_FAULT_MASK) >> 7;
	Fault_Status_Reg.FSO_GDF   = (regValue & DRV8343S_GDF_MASK) >> 6;
	Fault_Status_Reg.FSO_CPUV  = (regValue & DRV8343S_CPUV_MASK) >> 5;
	Fault_Status_Reg.FSO_UVLO  = (regValue & DRV8343S_UVLO_MASK) >> 4;
	Fault_Status_Reg.FSO_OCP   = (regValue & DRV8343S_OCP_MASK) >> 3;
	Fault_Status_Reg.FSO_OTW   = (regValue & DRV8343S_OTW_MASK) >> 2;
	Fault_Status_Reg.FSO_OTSD  = (regValue & DRV8343S_OTSD_MASK) >> 1;
	Fault_Status_Reg.FSO_OL_SHT = (regValue & DRV8343S_OL_SHT_MASK) >> 0;

    // Read Register 0x01
    regValue = drv8343_SPI_Read(SPI_REG_DIAG_STAT_A);

    Diag_status_A_Reg.DSA_FAULT 	= (regValue & DRV8343S_SA_OC_MASK) >> 7;
    Diag_status_A_Reg.DSA_SA_OC		= (regValue & DRV8343S_SHT_GND_A_MASK) >> 6;
    Diag_status_A_Reg.DSA_SHT_BAT_A	= (regValue & DRV8343S_SHT_BAT_A_MASK) >> 5;
    Diag_status_A_Reg.DSA_OL_PH_A	= (regValue & DRV8343S_OL_PH_A_MASK) >> 4;
    Diag_status_A_Reg.DSA_VGS_LA	= (regValue & DRV8343S_VGS_LA_MASK) >> 3;
    Diag_status_A_Reg.DSA_VGS_HA	= (regValue & DRV8343S_VGS_HA_MASK) >> 2;
    Diag_status_A_Reg.DSA_VDS_LA	= (regValue & DRV8343S_VDS_LA_MASK) >> 1;
    Diag_status_A_Reg.DSA_VDS_HA	= (regValue & DRV8343S_VDS_HA_MASK) >> 0;

	// Read Register 0x02
    regValue = drv8343_SPI_Read(SPI_REG_DIAG_STAT_B);

    Diag_status_B_Reg.DSB_FAULT 	= (regValue & DRV8343S_SB_OC_MASK) >> 7;
    Diag_status_B_Reg.DSB_SB_OC		= (regValue & DRV8343S_SHT_GND_B_MASK) >> 6;
    Diag_status_B_Reg.DSB_SHT_BAT_B	= (regValue & DRV8343S_SHT_BAT_B_MASK) >> 5;
    Diag_status_B_Reg.DSB_OL_PH_B	= (regValue & DRV8343S_OL_PH_B_MASK) >> 4;
    Diag_status_B_Reg.DSB_VGS_LB	= (regValue & DRV8343S_VGS_LB_MASK) >> 3;
    Diag_status_B_Reg.DSB_VGS_HB	= (regValue & DRV8343S_VGS_HB_MASK) >> 2;
    Diag_status_B_Reg.DSB_VDS_LB	= (regValue & DRV8343S_VDS_LB_MASK) >> 1;
    Diag_status_B_Reg.DSB_VDS_HB	= (regValue & DRV8343S_VDS_HB_MASK) >> 0;


    // Read Register 0x03
    regValue = drv8343_SPI_Read(SPI_REG_DIAG_STAT_C);

    Diag_status_C_Reg.DSC_FAULT 	= (regValue & DRV8343S_SC_OC_MASK) >> 7;
    Diag_status_C_Reg.DSC_SC_OC		= (regValue & DRV8343S_SHT_GND_C_MASK) >> 6;
    Diag_status_C_Reg.DSC_SHT_BAT_C	= (regValue & DRV8343S_SHT_BAT_C_MASK) >> 5;
    Diag_status_C_Reg.DSC_OL_PH_C	= (regValue & DRV8343S_OL_PH_C_MASK) >> 4;
    Diag_status_C_Reg.DSC_VGS_LC	= (regValue & DRV8343S_VGS_LC_MASK) >> 3;
    Diag_status_C_Reg.DSC_VGS_HC	= (regValue & DRV8343S_VGS_HC_MASK) >> 2;
    Diag_status_C_Reg.DSC_VDS_LC	= (regValue & DRV8343S_VDS_LC_MASK) >> 1;
    Diag_status_C_Reg.DSC_VDS_HC	= (regValue & DRV8343S_VDS_HC_MASK) >> 0;

    // Read Register 0x04

    regValue = drv8343_SPI_Read(SPI_REG_DRV_CTRL_1);

    Drv_Control_Reg1.DCI1_CLR_FLT   	=  (regValue & DRV8343S_CLR_FLT_MASK) >>7;
	Drv_Control_Reg1.DCI1_PWM_MODE  	=  (regValue & DRV8343S_PWM_MODE_MASK) >>4;
	Drv_Control_Reg1.DCI1_ONE_PWM_COM   =  (regValue & DRV8343S_ONE_PWM_COM_MASK) >>3;
	Drv_Control_Reg1.DCI1_ONE_PWM_DIR	=  (regValue & DRV8343S_ONE_PWM_DIR_MASK) >> 2;
	Drv_Control_Reg1.DCI1_ONE_PWM_BRK	=  (regValue & DRV8343S_ONE_PWM_BRK_MASK) >> 0;

		// Read Register 0x05
    regValue = drv8343_SPI_Read(SPI_REG_DRV_CTRL_2);
	Drv_Control_Reg2.DCI2_OTSD_MODE		=	(regValue & DRV8343S_OTSD_MODE_MASK) >> 7;
	Drv_Control_Reg2.DCI2_OLP_SHTS_DLY  =	(regValue & DRV8343S_OLP_SHTS_DLY_MASK) >> 5;
	Drv_Control_Reg2.DCI2_EN_SHT_TST    =	(regValue & DRV8343S_EN_SHT_TST_MASK) >> 4;
	Drv_Control_Reg2.DCI2_EN_OLP		=   (regValue & DRV8343S_EN_OLP_MASK) >> 3;
	Drv_Control_Reg2.DCI2_EN_OLA_C     	=	(regValue & DRV8343S_EN_OLA_C_MASK) >> 2;
	Drv_Control_Reg2.DCI2_EN_OLA_B		=	(regValue & DRV8343S_EN_OLA_B_MASK) >> 1;
	Drv_Control_Reg2.DCI2_EN_OLA_A      =	(regValue & DRV8343S_EN_OLA_A_MASK) >> 0;

	// Read Register 0x06
    regValue = drv8343_SPI_Read(SPI_REG_DRV_CTRL_3);

    Drv_Control_Reg3.DCI3_IDRIVEP_LA	=	(regValue & DRV8343S_IDRIVEP_LA_MASK) >> 4;
    Drv_Control_Reg3.DCI3_IDRIVEP_HA	=	(regValue & DRV8343S_IDRIVEP_HA_MASK) >> 0;

    // Read Register 0x07
    regValue = drv8343_SPI_Read(SPI_REG_DRV_CTRL_4);

    Drv_Control_Reg4.DCI4_IDRIVEP_LB	=	(regValue & DRV8343S_IDRIVEP_LB_MASK) >> 4;
    Drv_Control_Reg4.DCI4_IDRIVEP_HB	=	(regValue & DRV8343S_IDRIVEP_HB_MASK) >> 0;

    // Read Register 0x08
    regValue = drv8343_SPI_Read(SPI_REG_DRV_CTRL_5);

    Drv_Control_Reg5.DCI5_IDRIVEP_LC	=	(regValue & DRV8343S_IDRIVEP_LC_MASK) >> 4;
    Drv_Control_Reg5.DCI5_IDRIVEP_HC	=	(regValue & DRV8343S_IDRIVEP_HC_MASK) >> 0;


    // Read Register 0x09
    regValue = drv8343_SPI_Read(SPI_REG_DRV_CTRL_6);

    Drv_Control_Reg6.DCI6_VDS_LA	=	(regValue & DRV8343S_VDS_LVL_LA_MASK) >> 4;
    Drv_Control_Reg6.DCI6_VDS_HA	=	(regValue & DRV8343S_VDS_LVL_HA_MASK) >> 0;


    // Read Register 0x0A
    regValue = drv8343_SPI_Read(SPI_REG_DRV_CTRL_7);

    Drv_Control_Reg7.DCI7_VDS_LB	=	(regValue & DRV8343S_VDS_LVL_LB_MASK) >> 4;
    Drv_Control_Reg7.DCI7_VDS_HB	=	(regValue & DRV8343S_VDS_LVL_HB_MASK) >> 0;

    // Read Register 0x0B
    regValue = drv8343_SPI_Read(SPI_REG_DRV_CTRL_8);

    Drv_Control_Reg8.DCI8_VDS_LC	=	(regValue & DRV8343S_VDS_LVL_LC_MASK) >> 4;
    Drv_Control_Reg8.DCI8_VDS_HC	=	(regValue & DRV8343S_VDS_LVL_HC_MASK) >> 0;


    // Read Register 0x0C
    regValue = drv8343_SPI_Read(SPI_REG_DRV_CTRL_9);
    Drv_Control_Reg9.DCI9_COAST			=	(regValue & DRV8343S_COAST_MASK) >> 7;
    Drv_Control_Reg9.DCI9_TRETRY		=	(regValue & DRV8343S_TRETRY_MASK) >> 5;
    Drv_Control_Reg9.DCI9_DEAD_TIME		=	(regValue & DRV8343S_DEAD_TIME_MASK) >> 3;
    Drv_Control_Reg9.DCI9_TDRIVE_MAX	=	(regValue & DRV8343S_TDRIVE_MAX_MASK) >> 2;
    Drv_Control_Reg9.DCI9_TDRIVE		=	(regValue & DRV8343S_TDRIVE_MASK) >> 0;

    // Read Register 0x0D
    regValue = drv8343_SPI_Read(SPI_REG_DRV_CTRL_10);
    Drv_Control_Reg10.DCI10_LOCK		=	(regValue & DRV8343S_LOCK_MASK) >> 5;
    Drv_Control_Reg10.DCI10_DIS_CPUV	=	(regValue & DRV8343S_DIS_CPUV_MASK) >> 4;
    Drv_Control_Reg10.DCI10_DIS_GDF		=	(regValue & DRV8343S_DIS_GDF_MASK) >> 3;
    Drv_Control_Reg10.DCI10_OCP_DEG		=	(regValue & DRV8343S_OCP_DEG_MASK) >> 0;


    // Read Register 0x0E
    regValue = drv8343_SPI_Read(SPI_REG_DRV_CTRL_11);
    Drv_Control_Reg11.DCI11_RSVD_REG_10 	=	(regValue & DRV8343S_RSVD_REG_10_MASK) >> 7;
	Drv_Control_Reg11.DCI11_OTW_REP			=	(regValue & DRV8343S_OTW_REP_MASK) >> 6;
	Drv_Control_Reg11.DCI11_CBC				=	(regValue & DRV8343S_CBC_MASK) >> 5;
	Drv_Control_Reg11.DCI11_DIS_VDS_C		=	(regValue & DRV8343S_DIS_VDS_C_MASK) >> 4;
	Drv_Control_Reg11.DCI11_DIS_VDS_B		=	(regValue & DRV8343S_DIS_VDS_B_MASK) >> 3;
	Drv_Control_Reg11.DCI11_DIS_VDS_A		=	(regValue & DRV8343S_DIS_VDS_A_MASK) >> 2;
	Drv_Control_Reg11.DCI11_OCP_MODE		=	(regValue & DRV8343S_OCP_MODE_MASK) >> 0;


    // Read Register 0x0F
    regValue = drv8343_SPI_Read(SPI_REG_DRV_CTRL_12);
    Drv_Control_Reg12.DCI12_LS_REF			=	(regValue & DRV8343S_LS_REF_MASK) >> 7;
	Drv_Control_Reg12.DCI12_CSA_FET			=	(regValue & DRV8343S_CSA_FET_MASK) >> 6;
	Drv_Control_Reg12.DCI12_CSA_GAIN_C		=	(regValue & DRV8343S_CSA_GAIN_C_MASK) >> 4;
	Drv_Control_Reg12.DCI12_CSA_GAIN_B		=	(regValue & DRV8343S_CSA_GAIN_B_MASK) >> 2;
	Drv_Control_Reg12.DCI12_CSA_GAIN_A		=	(regValue & DRV8343S_CSA_GAIN_A_MASK) >> 0;


    // Read Register 0x10
    regValue = drv8343_SPI_Read(SPI_REG_DRV_CTRL_13);
    Drv_Control_Reg13.DCI13_CAL_MODE		=	(regValue & DRV8343S_CAL_MODE_MASK) >> 7;
	Drv_Control_Reg13.DCI13_VREF_DIV		=	(regValue & DRV8343S_VREF_DIV_MASK) >> 6;
	Drv_Control_Reg13.DCI13_SEN_LVL_C		=	(regValue & DRV8343S_SEN_LVL_C_MASK) >> 4;
	Drv_Control_Reg13.DCI13_SEN_LVL_B		=	(regValue & DRV8343S_SEN_LVL_B_MASK) >> 2;
	Drv_Control_Reg13.DCI13_SEN_LVL_A		=	(regValue & DRV8343S_SEN_LVL_A_MASK) >> 0;


	// Read Register 0x11
    regValue = drv8343_SPI_Read(SPI_REG_DRV_CTRL_14);
    Drv_Control_Reg14.DCI14_RSVD_REG_14		=	(regValue & DRV8343S_RSVD_REG_14_MASK) >> 6;
	Drv_Control_Reg14.DCI14_DIS_SEN_C		=	(regValue & DRV8343S_DIS_SEN_C_MASK) >> 5;
	Drv_Control_Reg14.DCI14_DIS_SEN_B		=	(regValue & DRV8343S_DIS_SEN_B_MASK) >> 4;
	Drv_Control_Reg14.DCI14_DIS_SEN_A		=	(regValue & DRV8343S_DIS_SEN_A_MASK) >> 3;
	Drv_Control_Reg14.DCI14_CSA_CAL_C		=	(regValue & DRV8343S_CSA_CAL_C_MASK) >> 2;
	Drv_Control_Reg14.DCI14_CSA_CAL_B		=	(regValue & DRV8343S_CSA_CAL_B_MASK) >> 1;
	Drv_Control_Reg14.DCI14_CSA_CAL_A		=	(regValue & DRV8343S_CSA_CAL_A_MASK) >> 0;

}

void drv83xx_Register_Init(void)
{
    Fault_Status_Reg.FSO_FAULT =  DRV8343S_FAULT;
	Fault_Status_Reg.FSO_GDF   =  DRV8343S_GDF;
	Fault_Status_Reg.FSO_CPUV  =  DRV8343S_CPUV;
	Fault_Status_Reg.FSO_UVLO  =  DRV8343S_UVLO;
	Fault_Status_Reg.FSO_OCP   =  DRV8343S_OCP;
	Fault_Status_Reg.FSO_OTW   =  DRV8343S_OTW;
	Fault_Status_Reg.FSO_OTSD  =  DRV8343S_OTSD;
	Fault_Status_Reg.FSO_OL_SHT =  DRV8343S_OL_SHT;



	// Set Register 0x01
    Diag_status_A_Reg.DSA_FAULT 	= DRV8343S_SA_OC;
    Diag_status_A_Reg.DSA_SA_OC		= DRV8343S_SHT_GND_A;
    Diag_status_A_Reg.DSA_SHT_BAT_A	= DRV8343S_SHT_BAT_A;
    Diag_status_A_Reg.DSA_OL_PH_A	= DRV8343S_OL_PH_A;
    Diag_status_A_Reg.DSA_VGS_LA	= DRV8343S_VGS_LA;
    Diag_status_A_Reg.DSA_VGS_HA	= DRV8343S_VGS_HA;
    Diag_status_A_Reg.DSA_VDS_LA	= DRV8343S_VDS_LA;
    Diag_status_A_Reg.DSA_VDS_HA	= DRV8343S_VDS_HA;

	// Set Register 0x02

    Diag_status_B_Reg.DSB_FAULT 	= DRV8343S_SB_OC;
    Diag_status_B_Reg.DSB_SB_OC		= DRV8343S_SHT_GND_B;
    Diag_status_B_Reg.DSB_SHT_BAT_B	= DRV8343S_SHT_BAT_B;
    Diag_status_B_Reg.DSB_OL_PH_B	= DRV8343S_OL_PH_B;
    Diag_status_B_Reg.DSB_VGS_LB	= DRV8343S_VGS_LB;
    Diag_status_B_Reg.DSB_VGS_HB	= DRV8343S_VGS_HB;
    Diag_status_B_Reg.DSB_VDS_LB	= DRV8343S_VDS_LB;
    Diag_status_B_Reg.DSB_VDS_HB	= DRV8343S_VDS_HB;

	// Set Register 0x03
    Diag_status_C_Reg.DSC_FAULT 	= DRV8343S_SC_OC;
    Diag_status_C_Reg.DSC_SC_OC		= DRV8343S_SHT_GND_C;
    Diag_status_C_Reg.DSC_SHT_BAT_C	= DRV8343S_SHT_BAT_C;
    Diag_status_C_Reg.DSC_OL_PH_C	= DRV8343S_OL_PH_C;
    Diag_status_C_Reg.DSC_VGS_LC	= DRV8343S_VGS_LC;
    Diag_status_C_Reg.DSC_VGS_HC	= DRV8343S_VGS_HC;
    Diag_status_C_Reg.DSC_VDS_LC	= DRV8343S_VDS_LC;
    Diag_status_C_Reg.DSC_VDS_HC	= DRV8343S_VDS_HC;

	// Set Register 0x04
    Drv_Control_Reg1.DCI1_CLR_FLT   	=  DRV8343S_CLR_FLT;
	Drv_Control_Reg1.DCI1_PWM_MODE  	=  DRV8343S_PWM_MODE;
	Drv_Control_Reg1.DCI1_ONE_PWM_COM   =  DRV8343S_ONE_PWM_COM;
	Drv_Control_Reg1.DCI1_ONE_PWM_DIR	=  DRV8343S_ONE_PWM_DIR;
	Drv_Control_Reg1.DCI1_ONE_PWM_BRK	=  DRV8343S_ONE_PWM_BRK;

    // Set Register 0x05
	Drv_Control_Reg2.DCI2_OTSD_MODE		=	DRV8343S_OTSD_MODE;
	Drv_Control_Reg2.DCI2_OLP_SHTS_DLY  =	DRV8343S_OLP_SHTS_DLY;
	Drv_Control_Reg2.DCI2_EN_SHT_TST    =	DRV8343S_EN_SHT_TST;
	Drv_Control_Reg2.DCI2_EN_OLP		=   DRV8343S_EN_OLP;
	Drv_Control_Reg2.DCI2_EN_OLA_C     	=	DRV8343S_EN_OLA_C;
	Drv_Control_Reg2.DCI2_EN_OLA_B		=	DRV8343S_EN_OLA_B;
	Drv_Control_Reg2.DCI2_EN_OLA_A      =	DRV8343S_EN_OLA_A;

		// Set Register 0x06
    Drv_Control_Reg3.DCI3_IDRIVEP_LA	=	DRV_Control.Idrive_Setting;
    Drv_Control_Reg3.DCI3_IDRIVEP_HA	=	DRV_Control.Idrive_Setting;

	// Set Register 0x07
    Drv_Control_Reg4.DCI4_IDRIVEP_LB	=	DRV_Control.Idrive_Setting;
    Drv_Control_Reg4.DCI4_IDRIVEP_HB	=	DRV_Control.Idrive_Setting;

	// Set Register 0x08
    Drv_Control_Reg5.DCI5_IDRIVEP_LC	=	DRV_Control.Idrive_Setting;
    Drv_Control_Reg5.DCI5_IDRIVEP_HC	=	DRV_Control.Idrive_Setting;

	// Set Register 0x09
    Drv_Control_Reg6.DCI6_VDS_LA	=	DRV8343S_VDS_LVL_LA;
    Drv_Control_Reg6.DCI6_VDS_HA	=	DRV8343S_VDS_LVL_HA;

	// Set Register 0x0A
    Drv_Control_Reg7.DCI7_VDS_LB	=	DRV8343S_VDS_LVL_LB;
    Drv_Control_Reg7.DCI7_VDS_HB	=	DRV8343S_VDS_LVL_HB;

	// Set Register 0x0B
    Drv_Control_Reg8.DCI8_VDS_LC	=	DRV8343S_VDS_LVL_LC;
    Drv_Control_Reg8.DCI8_VDS_HC	=	DRV8343S_VDS_LVL_HC;
    // Read Register 0x0C

    Drv_Control_Reg9.DCI9_COAST			=	DRV8343S_COAST;
    Drv_Control_Reg9.DCI9_TRETRY		=	DRV8343S_TRETRY;
    Drv_Control_Reg9.DCI9_DEAD_TIME		=	DRV8343S_DEAD_TIME;
    Drv_Control_Reg9.DCI9_TDRIVE_MAX	=	DRV8343S_TDRIVE_MAX;
    Drv_Control_Reg9.DCI9_TDRIVE		=	DRV_Control.Tdrive_Setting;

    // Read Register 0x0D

    Drv_Control_Reg10.DCI10_LOCK		=	DRV8343S_LOCK;
    Drv_Control_Reg10.DCI10_DIS_CPUV	=	DRV8343S_DIS_CPUV;
    Drv_Control_Reg10.DCI10_DIS_GDF		=	DRV8343S_DIS_GDF;
    Drv_Control_Reg10.DCI10_OCP_DEG		=	DRV8343S_OCP_DEG;

    // Read Register 0x0E

    Drv_Control_Reg11.DCI11_RSVD_REG_10 	=	DRV8343S_RSVD_REG_10;
	Drv_Control_Reg11.DCI11_OTW_REP			=	DRV8343S_OTW_REP;
	Drv_Control_Reg11.DCI11_CBC				=	DRV8343S_CBC;
	Drv_Control_Reg11.DCI11_DIS_VDS_C		=	DRV8343S_DIS_VDS_C;
	Drv_Control_Reg11.DCI11_DIS_VDS_B		=	DRV8343S_DIS_VDS_B;
	Drv_Control_Reg11.DCI11_DIS_VDS_A		=	DRV8343S_DIS_VDS_A;
	Drv_Control_Reg11.DCI11_OCP_MODE		=	DRV8343S_OCP_MODE;


    // Read Register 0x0F

    Drv_Control_Reg12.DCI12_LS_REF			=	DRV8343S_LS_REF;
	Drv_Control_Reg12.DCI12_CSA_FET			=	DRV8343S_CSA_FET;
	Drv_Control_Reg12.DCI12_CSA_GAIN_C		=	DRV8343S_CSA_GAIN_C;
	Drv_Control_Reg12.DCI12_CSA_GAIN_B		=	DRV8343S_CSA_GAIN_B;
	Drv_Control_Reg12.DCI12_CSA_GAIN_A		=	DRV8343S_CSA_GAIN_A;

    // Read Register 0x10

    Drv_Control_Reg13.DCI13_CAL_MODE		=	DRV8343S_CAL_MODE;
	Drv_Control_Reg13.DCI13_VREF_DIV		=	DRV8343S_VREF_DIV;
	Drv_Control_Reg13.DCI13_SEN_LVL_C		=	DRV8343S_SEN_LVL_C;
	Drv_Control_Reg13.DCI13_SEN_LVL_B		=	DRV8343S_SEN_LVL_B;
	Drv_Control_Reg13.DCI13_SEN_LVL_A		=	DRV8343S_SEN_LVL_A;

    // Read Register 0x11

    Drv_Control_Reg14.DCI14_RSVD_REG_14		=	DRV8343S_RSVD_REG_14;
	Drv_Control_Reg14.DCI14_DIS_SEN_C		=	DRV8343S_DIS_SEN_C;
	Drv_Control_Reg14.DCI14_DIS_SEN_B		=	DRV8343S_DIS_SEN_B;
	Drv_Control_Reg14.DCI14_DIS_SEN_A		=	DRV8343S_DIS_SEN_A;
	Drv_Control_Reg14.DCI14_CSA_CAL_C		=	DRV8343S_CSA_CAL_C;
	Drv_Control_Reg14.DCI14_CSA_CAL_B		=	DRV8343S_CSA_CAL_B;
	Drv_Control_Reg14.DCI14_CSA_CAL_A		=	DRV8343S_CSA_CAL_A;

}
void drv83xx_Register_Write(void)
{
    uint16_t regValue;
	// Read Register 0x00

	regValue = (Fault_Status_Reg.FSO_FAULT << 7) |
				(Fault_Status_Reg.FSO_GDF    << 6) |
				(Fault_Status_Reg.FSO_CPUV   << 5) |
				(Fault_Status_Reg.FSO_UVLO   << 4) |
				(Fault_Status_Reg.FSO_OCP   << 3) |
				(Fault_Status_Reg.FSO_OTW   << 2) |
				(Fault_Status_Reg.FSO_OTSD   << 1) |
				(Fault_Status_Reg.FSO_OL_SHT  << 0);

	drv8343_SPI_Write(SPI_REG_FAULT_STAT,regValue);
	Reg_Map_Cache.FAULT_STAT = regValue;

    // Read Register 0x01

	regValue = (Diag_status_A_Reg.DSA_FAULT << 7) |
				(Diag_status_A_Reg.DSA_SA_OC << 6) |
				(Diag_status_A_Reg.DSA_SHT_BAT_A << 5) |
				(Diag_status_A_Reg.DSA_OL_PH_A	 << 4) |
				(Diag_status_A_Reg.DSA_VGS_LA	 << 3) |
				(Diag_status_A_Reg.DSA_VGS_HA	 << 2) |
				(Diag_status_A_Reg.DSA_VDS_LA	 << 1)|
				(Diag_status_A_Reg.DSA_VDS_HA	 << 0);
	drv8343_SPI_Write(SPI_REG_DIAG_STAT_A,regValue);
	Reg_Map_Cache.DIAG_STAT_A = regValue;

    // Read Register 0x02
    regValue = ( Diag_status_B_Reg.DSB_FAULT 	<< 7) |
				(Diag_status_B_Reg.DSB_SB_OC		 << 6) |
				(Diag_status_B_Reg.DSB_SHT_BAT_B	<< 5) |
				(Diag_status_B_Reg.DSB_OL_PH_B	 << 4) |
				(Diag_status_B_Reg.DSB_VGS_LB	 << 3) |
				(Diag_status_B_Reg.DSB_VGS_HB	 << 2) |
				(Diag_status_B_Reg.DSB_VDS_LB	 << 1) |
				(Diag_status_B_Reg.DSB_VDS_HB	 << 0);
	 drv8343_SPI_Write(SPI_REG_DIAG_STAT_B,regValue);
	Reg_Map_Cache.DIAG_STAT_B = regValue;

    // Read Register 0x03
    regValue = (Diag_status_C_Reg.DSC_FAULT 	 << 7) |
				 (Diag_status_C_Reg.DSC_SC_OC	<< 6) |
				 (Diag_status_C_Reg.DSC_SHT_BAT_C << 5) |
				 (Diag_status_C_Reg.DSC_OL_PH_C	 << 4) |
				 (Diag_status_C_Reg.DSC_VGS_LC	 << 3) |
				 (Diag_status_C_Reg.DSC_VGS_HC	 << 2) |
				 (Diag_status_C_Reg.DSC_VDS_LC	 << 1) |
				 (Diag_status_C_Reg.DSC_VDS_HC	 << 0);
	 drv8343_SPI_Write(SPI_REG_DIAG_STAT_C,regValue);
	Reg_Map_Cache.DIAG_STAT_C = regValue;

    // Read Register 0x04
	 regValue = (Drv_Control_Reg1.DCI1_CLR_FLT   	 <<7) |
				(Drv_Control_Reg1.DCI1_PWM_MODE  	 <<4) |
				(Drv_Control_Reg1.DCI1_ONE_PWM_COM   <<3) |
				(Drv_Control_Reg1.DCI1_ONE_PWM_DIR	 << 2)|
				(Drv_Control_Reg1.DCI1_ONE_PWM_BRK	 << 0);
	 drv8343_SPI_Write(SPI_REG_DRV_CTRL_1,regValue);
		Reg_Map_Cache.DRV_CTRL_IC1 = regValue;

    // Read Register 0x05
	 regValue = (Drv_Control_Reg2.DCI2_OTSD_MODE	 << 7) |
				(Drv_Control_Reg2.DCI2_OLP_SHTS_DLY  << 5) |
				(Drv_Control_Reg2.DCI2_EN_SHT_TST    << 4) |
				(Drv_Control_Reg2.DCI2_EN_OLP		 << 3) |
				(Drv_Control_Reg2.DCI2_EN_OLA_C      << 2) |
				(Drv_Control_Reg2.DCI2_EN_OLA_B		 << 1) |
				(Drv_Control_Reg2.DCI2_EN_OLA_A      << 0);
	 drv8343_SPI_Write(SPI_REG_DRV_CTRL_2,regValue);
	 Reg_Map_Cache.DRV_CTRL_IC2 = regValue;


	// Read Register 0x06
    regValue = (Drv_Control_Reg3.DCI3_IDRIVEP_LA	<< 4) |
    			(Drv_Control_Reg3.DCI3_IDRIVEP_HA	 << 0);
	drv8343_SPI_Write(SPI_REG_DRV_CTRL_3,regValue);
	Reg_Map_Cache.DRV_CTRL_IC3 = regValue;

    // Read Register 0x07

    regValue = (Drv_Control_Reg4.DCI4_IDRIVEP_LB	 << 4) |
    		   (Drv_Control_Reg4.DCI4_IDRIVEP_HB	<< 0);
	drv8343_SPI_Write(SPI_REG_DRV_CTRL_4,regValue);
    Reg_Map_Cache.DRV_CTRL_IC4 = regValue;

    // Read Register 0x08
	regValue = ((Drv_Control_Reg5.DCI5_IDRIVEP_LC	 << 4) |
				   (Drv_Control_Reg5.DCI5_IDRIVEP_HC	 << 0));
	drv8343_SPI_Write(SPI_REG_DRV_CTRL_5,regValue);
	Reg_Map_Cache.DRV_CTRL_IC5 = regValue;

    // Read Register 0x09
    regValue = (Drv_Control_Reg6.DCI6_VDS_LA	 << 4) |
    		   (Drv_Control_Reg6.DCI6_VDS_HA	 << 0);
	 drv8343_SPI_Write(SPI_REG_DRV_CTRL_6,regValue);
	 Reg_Map_Cache.DRV_CTRL_IC6= regValue;

    // Read Register 0x0A

    regValue = (Drv_Control_Reg7.DCI7_VDS_LB	 << 4) |
    		   (Drv_Control_Reg7.DCI7_VDS_HB	 << 0);
	 drv8343_SPI_Write(SPI_REG_DRV_CTRL_7,regValue);
	 Reg_Map_Cache.DRV_CTRL_IC7 = regValue;

	// Read Register 0x0B

	regValue = (Drv_Control_Reg8.DCI8_VDS_LC	 << 4) |
				(Drv_Control_Reg8.DCI8_VDS_HC	 << 0);
	 drv8343_SPI_Write(SPI_REG_DRV_CTRL_8,regValue);
	 Reg_Map_Cache.DRV_CTRL_IC8 = regValue;

	// Read Register 0x0C
    regValue = ( Drv_Control_Reg9.DCI9_COAST		 << 7) |
				(Drv_Control_Reg9.DCI9_TRETRY		 << 5) |
				(Drv_Control_Reg9.DCI9_DEAD_TIME	 << 3) |
				(Drv_Control_Reg9.DCI9_TDRIVE_MAX	 << 2) |
				(Drv_Control_Reg9.DCI9_TDRIVE		 << 0) ;
	 drv8343_SPI_Write(SPI_REG_DRV_CTRL_9,regValue);
	 Reg_Map_Cache.DRV_CTRL_IC9 = regValue;

		// Read Register 0x0D
    regValue = (Drv_Control_Reg10.DCI10_LOCK	     << 5) |
				(Drv_Control_Reg10.DCI10_DIS_CPUV	 << 4) |
				(Drv_Control_Reg10.DCI10_DIS_GDF	 << 3) |
				(Drv_Control_Reg10.DCI10_OCP_DEG	 << 0);
	 drv8343_SPI_Write(SPI_REG_DRV_CTRL_10,regValue);
   	 Reg_Map_Cache.DRV_CTRL_IC10 = regValue;

	 // Read Register 0x0E
	 regValue = (Drv_Control_Reg11.DCI11_RSVD_REG_10 	 << 7) |
				(Drv_Control_Reg11.DCI11_OTW_REP		 << 6) |
				(Drv_Control_Reg11.DCI11_CBC		     << 5) |
				(Drv_Control_Reg11.DCI11_DIS_VDS_C		 << 4) |
				(Drv_Control_Reg11.DCI11_DIS_VDS_B		 << 3) |
				(Drv_Control_Reg11.DCI11_DIS_VDS_A		 << 2) |
				(Drv_Control_Reg11.DCI11_OCP_MODE		 << 0);
	 drv8343_SPI_Write(SPI_REG_DRV_CTRL_11,regValue);
	 Reg_Map_Cache.DRV_CTRL_IC11 = regValue;


	// Read Register 0x0F
    regValue = (Drv_Control_Reg12.DCI12_LS_REF			 << 7) |
				(Drv_Control_Reg12.DCI12_CSA_FET		 << 6) |
				(Drv_Control_Reg12.DCI12_CSA_GAIN_C		 << 4) |
				(Drv_Control_Reg12.DCI12_CSA_GAIN_B		 << 2) |
				(Drv_Control_Reg12.DCI12_CSA_GAIN_A		 << 0);
	 drv8343_SPI_Write(SPI_REG_DRV_CTRL_12,regValue);
	 Reg_Map_Cache.DRV_CTRL_IC12 = regValue;

	 // Read Register 0x10
	 regValue = (Drv_Control_Reg13.DCI13_CAL_MODE	<< 7) |
				(Drv_Control_Reg13.DCI13_VREF_DIV		 << 6) |
				(Drv_Control_Reg13.DCI13_SEN_LVL_C		 << 4) |
				(Drv_Control_Reg13.DCI13_SEN_LVL_B		 << 2) |
				(Drv_Control_Reg13.DCI13_SEN_LVL_A		 << 0);
	 drv8343_SPI_Write(SPI_REG_DRV_CTRL_13, regValue);
	 Reg_Map_Cache.DRV_CTRL_IC13 = regValue;

	 // Read Register 0x11
	regValue = (Drv_Control_Reg14.DCI14_RSVD_REG_14	<< 6) |
				(Drv_Control_Reg14.DCI14_DIS_SEN_C		 << 5) |
				(Drv_Control_Reg14.DCI14_DIS_SEN_B		 << 4) |
				(Drv_Control_Reg14.DCI14_DIS_SEN_A		 << 3) |
				(Drv_Control_Reg14.DCI14_CSA_CAL_C		 << 2) |
				(Drv_Control_Reg14.DCI14_CSA_CAL_B		 << 1) |
				(Drv_Control_Reg14.DCI14_CSA_CAL_A		 << 0);    // Write Register 0x02
	 drv8343_SPI_Write(SPI_REG_DRV_CTRL_14,regValue);
	 Reg_Map_Cache.DRV_CTRL_IC14 = regValue;
}
/*function
 * drv83xx_regRestoreFromCache()
 * Restores the device register values by rewriting them with the cached values.
 * */
void drv83xx_regRestoreFromCache()
{
		/* Write all the cached register values to the device */
		drv8343_SPI_Write(SPI_REG_FAULT_STAT, Reg_Map_Cache.FAULT_STAT);
		drv8343_SPI_Write(SPI_REG_DIAG_STAT_A, Reg_Map_Cache.DIAG_STAT_A);
		drv8343_SPI_Write(SPI_REG_DIAG_STAT_B, Reg_Map_Cache.DIAG_STAT_B);
		drv8343_SPI_Write(SPI_REG_DIAG_STAT_C, Reg_Map_Cache.DIAG_STAT_C);
		drv8343_SPI_Write(SPI_REG_DRV_CTRL_1, Reg_Map_Cache.DRV_CTRL_IC1);
		drv8343_SPI_Write(SPI_REG_DRV_CTRL_2, Reg_Map_Cache.DRV_CTRL_IC2);
		drv8343_SPI_Write(SPI_REG_DRV_CTRL_3, Reg_Map_Cache.DRV_CTRL_IC3);
		drv8343_SPI_Write(SPI_REG_DRV_CTRL_4, Reg_Map_Cache.DRV_CTRL_IC4);
		drv8343_SPI_Write(SPI_REG_DRV_CTRL_5, Reg_Map_Cache.DRV_CTRL_IC5);
		drv8343_SPI_Write(SPI_REG_DRV_CTRL_6, Reg_Map_Cache.DRV_CTRL_IC6);
		drv8343_SPI_Write(SPI_REG_DRV_CTRL_7, Reg_Map_Cache.DRV_CTRL_IC7);
		drv8343_SPI_Write(SPI_REG_DRV_CTRL_8, Reg_Map_Cache.DRV_CTRL_IC8);
		drv8343_SPI_Write(SPI_REG_DRV_CTRL_9, Reg_Map_Cache.DRV_CTRL_IC9);
		drv8343_SPI_Write(SPI_REG_DRV_CTRL_10, Reg_Map_Cache.DRV_CTRL_IC10);
		drv8343_SPI_Write(SPI_REG_DRV_CTRL_11, Reg_Map_Cache.DRV_CTRL_IC11);
		drv8343_SPI_Write(SPI_REG_DRV_CTRL_12, Reg_Map_Cache.DRV_CTRL_IC12);
		drv8343_SPI_Write(SPI_REG_DRV_CTRL_13, Reg_Map_Cache.DRV_CTRL_IC13);
		drv8343_SPI_Write(SPI_REG_DRV_CTRL_14, Reg_Map_Cache.DRV_CTRL_IC14);
		drv8343_SPI_Write(SPI_REG_DRV_CTRL_15, Reg_Map_Cache.DRV_CTRL_IC15);

}

/*function
 * drv83xx_CacheToReg(unsigned char address)
 * Caches the device register value in the firmware
 * */
unsigned int drv83xx_cachetoReg(unsigned char address)
{
	unsigned int regValue;
	switch (address)
	{
		case 0:
			regValue = Reg_Map_Cache.FAULT_STAT;
			break;

		case 1:
			regValue = Reg_Map_Cache.DIAG_STAT_A;
			break;

		case 2:
			regValue = Reg_Map_Cache.DIAG_STAT_B;
			break;
		case 3:
			regValue = Reg_Map_Cache.DIAG_STAT_C;
			break;

		case 4:
			regValue = Reg_Map_Cache.DRV_CTRL_IC1;
			break;

		case 5:
			regValue = Reg_Map_Cache.DRV_CTRL_IC2;
			break;

		case 6:
			regValue = Reg_Map_Cache.DRV_CTRL_IC3;
			break;
		case 7:
			regValue = Reg_Map_Cache.DRV_CTRL_IC4;
			break;
		case 8:
			regValue = Reg_Map_Cache.DRV_CTRL_IC5;
			break;
		case 9:
			regValue = Reg_Map_Cache.DRV_CTRL_IC6;
			break;
		case 10:
			regValue = Reg_Map_Cache.DRV_CTRL_IC7;
			break;
		case 11:
			regValue = Reg_Map_Cache.DRV_CTRL_IC8;
			break;
		case 12:
			regValue = Reg_Map_Cache.DRV_CTRL_IC9;
			break;
		case 13:
			regValue = Reg_Map_Cache.DRV_CTRL_IC10;
			break;
		case 14:
			regValue = Reg_Map_Cache.DRV_CTRL_IC11;
			break;
		case 15:
			regValue = Reg_Map_Cache.DRV_CTRL_IC12;
			break;
		case 16:
			regValue = Reg_Map_Cache.DRV_CTRL_IC13;
			break;
		case 17:
			regValue = Reg_Map_Cache.DRV_CTRL_IC14;
			break;
		case 18:
			regValue = Reg_Map_Cache.DRV_CTRL_IC15;
			break;

		default:
			break;
	}
	return regValue;
}
/*function
 * drv83xx_regToCache(unsigned char address)
 * Caches the device register value in the firmware
 * */
void drv83xx_regToCache(unsigned char address, unsigned int regValue)
{
	switch (address)
	{
		case 0:
			Reg_Map_Cache.FAULT_STAT = regValue;
			break;

		case 1:
			Reg_Map_Cache.DIAG_STAT_A = regValue;
			break;

		case 2:
			Reg_Map_Cache.DIAG_STAT_B = regValue;
			break;

		case 3:
			Reg_Map_Cache.DIAG_STAT_C = regValue;
			break;

		case 4:
			Reg_Map_Cache.DRV_CTRL_IC1 = regValue;
			break;

		case 5:
			Reg_Map_Cache.DRV_CTRL_IC2 = (regValue & ~(DRV8343S_EN_OLP_MASK));
			break;

		case 6:
			Reg_Map_Cache.DRV_CTRL_IC3 = regValue;
			break;
		case 7:
			Reg_Map_Cache.DRV_CTRL_IC4 = regValue;
			break;

		case 8:
			Reg_Map_Cache.DRV_CTRL_IC5 = regValue;
			break;

		case 9:
			Reg_Map_Cache.DRV_CTRL_IC6 = regValue;
			break;

		case 10:
			Reg_Map_Cache.DRV_CTRL_IC7 = regValue;
			break;

		case 11:
			Reg_Map_Cache.DRV_CTRL_IC8 = regValue;
			break;

		case 12:
			Reg_Map_Cache.DRV_CTRL_IC9 = regValue;
			break;

		case 13:
			Reg_Map_Cache.DRV_CTRL_IC10 = regValue;
			break;

		case 14:
			Reg_Map_Cache.DRV_CTRL_IC11 = regValue;
			break;

		case 15:
			Reg_Map_Cache.DRV_CTRL_IC12 = regValue;
			break;

		case 16:
			Reg_Map_Cache.DRV_CTRL_IC13 = regValue;
			break;

		case 17:
			Reg_Map_Cache.DRV_CTRL_IC14 = regValue;
			break;

		case 18:
			Reg_Map_Cache.DRV_CTRL_IC15 = regValue;
			break;


		default:
			break;
	}
}
unsigned short drv8343_SPI_Read(uint8_t regAddr)
{
	uint16_t data;
	uint8_t  value;
	data = SPI_ReadWord(regAddr);
	value = data & 0x00FF;                  // DRV8343 is a 8 bit data
    return value;
}
void drv8343_SPI_Write(uint8_t regAddr, uint8_t data)
{
    uint8_t dataLSB , dataMSB;
    dataMSB = regAddr;
    dataLSB = data;

    SPI_WriteTwoBytes(dataLSB , dataMSB);
}
